@@ -38,7 +38,15 @@ class TomlControlKeywords(Enum):
     """
     DEVICE="device"
     DEVICES="devices"
+    VARACTOR="varactor"
+    VARACTORS="varactors"
+    SWITCH="switch"
+    SWITCHES="switches"
     TYPE="type"
+    RON="ron" # target on resistance for the switch
+    CVAR="cvar" # target varactor capacitance
+    VGS="vgs"
+    VSG="vsg"
     VDS="vds"
     VSD="vsd"
     GMOVERID="gmoverid"
@@ -79,7 +87,7 @@ class TomlSpitVariables(Enum):
     CGB="cgb"
     CSB="csb"
     CDB="cdb"
-    C_VARACTOR="cvar"
+    CVAR="cvar" # actual varactor capacitance
     R_ON="ron"
     REGION="region"
     ALL="all"
@@ -92,7 +100,7 @@ class TomlControlType(Enum):
     NMOS="nch"
     PMOS="pch"
     
-class MosCell(object):
+class MosCell:
     """_summary_
     Object implementing the CMOS device
     control and main spice parameters
@@ -117,6 +125,8 @@ class MosCell(object):
         "cgb",
         "csb",
         "cdb",
+        "cdep",
+        "cgg",
         "cvar",
         "gm",
         "gmbs",
@@ -126,7 +136,7 @@ class MosCell(object):
         "fosc",
         "ron",
         "region",
-        "vdsat"
+        "vdsat",
     ]
     __UNITS__={
         "name":"",
@@ -135,7 +145,7 @@ class MosCell(object):
         "vsd":Units.VOLTAGE.value,
         "vsb":Units.VOLTAGE.value,
         "vbs":Units.VOLTAGE.value,
-        "gmoverid":Units.SIEMENS.value+Units.AMPERE.value+"^-1",
+        "gmoverid":Units.VOLTAGE.value+"^-1",
         "l":Units.METER.value,
         "id":Units.AMPERE.value,
         "vgs":Units.VOLTAGE.value,
@@ -146,16 +156,18 @@ class MosCell(object):
         "cgb":Units.FARAD.value,
         "csb":Units.FARAD.value,
         "cdb":Units.FARAD.value,
+        "cgg":Units.FARAD.value,
+        "cdep":Units.FARAD.value,
         "cvar":Units.FARAD.value,
         "gmbs":Units.SIEMENS.value,
         "gm":Units.SIEMENS.value,
         "gds":Units.SIEMENS.value,
-        "self_gain":" ",
+        "self_gain":"VV^-01",
         "ft":Units.HERTZ.value,
         "fosc":Units.HERTZ.value,
         "ron":Units.OHM.value,
-        "region":" ",
-        "vdsat":Units.VOLTAGE.value
+        "region":"",
+        "vdsat":Units.VOLTAGE.value,
     }
     def __init__(
         self,
@@ -165,11 +177,19 @@ class MosCell(object):
         vsb:float=0.0,
         gmoverid:float=1.0,
         l:float=30*Scale.NANO.value[1],
-        id:float=None
+        id:float=0.0,
+        vgs:float=0.0,
+        ron:float=0.0,
+        cvar:float =0.0
     ):
         self.name=name
         self.type=type
         # input vars
+        # varactor 
+        self.cvar=cvar if bool(cvar) else None
+        #switch
+        self.ron = ron if bool(ron) else None
+        # cell
         self.vds:float=vds
         self.vsd:float=-vds
         self.vsb:float=vsb
@@ -178,7 +198,7 @@ class MosCell(object):
         self.l:float=l
         self.id:float=id
         # output vars
-        self.vgs:float=None
+        self.vgs:float=vgs if bool(vgs) else None
         self.vsg:float=-self.vgs if bool(self.vgs) else None
         self.w:float=None
         self.cgs:float=None
@@ -186,16 +206,16 @@ class MosCell(object):
         self.cgb:float=None
         self.csb:float=None
         self.cdb:float=None
-        self.cvar:float=None
         self.gmbs:float=None
         self.gm:float=None
         self.gds:float=None
         self.self_gain:float=None
         self.ft:float=None
         self.fosc:float=None
-        self.ron:float=None
         self.region:float=None
         self.vdsat:float=None
+        self.cdep:float=None
+        self.cgg:float=None
         
         
     def __str__(self)->str:
@@ -211,7 +231,7 @@ class MosCell(object):
         return result
 
     def __parse_data__(self, key:str, val)->None:
-        if key == TomlControlKeywords.DEVICE.value:
+        if key == TomlControlKeywords.DEVICE.value or key == TomlControlKeywords.VARACTOR.value or key == TomlControlKeywords.SWITCH:
             if type(val) != str:
                 raise ValueError("Device name must be a string")
             self.name = val
@@ -221,6 +241,42 @@ class MosCell(object):
             if val not in [TomlControlType.NMOS.value, TomlControlType.PMOS.value]:
                 raise ValueError(f"Device type must be \"{TomlControlType.NMOS.value}\" or \"{TomlControlType.PMOS.value}\"")
             self.type = val
+        elif key == TomlControlKeywords.CVAR.value:
+            if type(val) not in [float, str, int]:
+                raise ValueError("Varactor Cap. CVAR must be parsed as a float, integer or string")
+            if type(val) == str:
+                self.cvar = stof(val)
+            else:
+                self.cvar = float(val)
+        elif key == TomlControlKeywords.RON.value:
+            if type(val) not in [float, str, int]:
+                raise ValueError("Switch On Resistance RON must be parsed as a float, integer or string")
+            if type(val) == str:
+                self.ron = stof(val)
+            else:
+                self.ron = float(val)
+        elif key == TomlControlKeywords.VGS.value:
+            if self.type != TomlControlType.NMOS.value:
+                raise ValueError("VGS value to be specified is only valid for NMOS devices")
+            if type(val) not in [float, str, int]:
+                raise ValueError("VGS value must parsed as a float, integer or string")
+            if type(val) == str:
+                self.vgs = stof(val)
+                self.vsg = -self.vgs
+            else:
+                self.vgs = float(val)
+                self.vsg = -self.vgs
+        elif key == TomlControlKeywords.VSG.value:
+            if self.type != TomlControlType.PMOS.value:
+                raise ValueError("VSG value to be specified is only valid for PMOS devices")
+            if type(val) not in [float, str, int]:
+                raise ValueError("VSG value must parsed as a float, integer or string")
+            if type(val) == str:
+                self.vsg = stof(val)
+                self.vgs = -self.vsg
+            else:
+                self.vsg = float(val)
+                self.vgs = -self.vsg
         elif key == TomlControlKeywords.VDS.value:
             if self.type != TomlControlType.NMOS.value:
                 raise ValueError("VDS value to be specified is only valid for NMOS devices")
@@ -269,9 +325,9 @@ class MosCell(object):
             if type(val) not in [float, str, int]:
                 raise ValueError("Gm/Id value must parsed as a float, integer or string")
             if type(val) == str:
-                self.gm_id = stof(val)
+                self.gmoverid = stof(val)
             else:
-                self.gm_id = float(val)
+                self.gmoverid = float(val)
         elif key == TomlControlKeywords.LENGTH.value:
             if type(val) not in [float, str]:
                 raise ValueError("Channel Length (L) value must parsed as a float or string")
@@ -290,20 +346,22 @@ class MosCell(object):
             raise ValueError(f"{key} is not a valid control variable for a CMOS device")
                 
         
-class Devices(object):
+class Devices:
     """_summary_
     A class object ot save all the devices to which
     the gm/id sizing process will be applied, and the output
     specifications of each device
     """
-    __slots__=["devices", "spits", "plot", "output_dir"]
+    __slots__=["devices", "varactors", "switches", "spits", "plot", "output_dir"]
     def __init__(self):
         self.devices={}
+        self.varactors={}
+        self.switches={}
         self.spits=defaultdict(list)
         self.plot=False
         self.output_dir=None
     
-    def __data_frame__(self)->DataFrame:
+    def __data_frame__(self, dev_type="cell")->DataFrame:
         """_summary_
         Converts the device dictionary to a Pandas dataframe
         Returns:
@@ -311,20 +369,70 @@ class Devices(object):
         """
         vars = [var for var in dir(MosCell) if not var.startswith('__')]
         columns = [var+"["+MosCell.__UNITS__[var]+"]" for var in vars]
-        data = defaultdict(list)
-        for device in self.devices.values():
-            for i,var in enumerate(vars):
-                data[columns[i]].append(getattr(device, var))
-        order = ["name", "type", "vds", "vsb", "gm_id", "l", "w", "id"]
-        order = [var+"["+MosCell.__UNITS__[var]+"]" for var in order]
-        order = order+[var for var in columns if var not in order]
-        return DataFrame(data, index=self.devices.keys())[order]
+        data = {k:[] for k in columns}
+        index = None
+        if dev_type == "cell":
+            for device in self.devices.values():
+                for i,var in enumerate(vars):
+                    data[columns[i]].append(getattr(device, var))
+            order = ["name", "type", "vds", "vsb", "gmoverid", "l", "w", "id"]
+            order = [var+"["+MosCell.__UNITS__[var]+"]" for var in order]
+            order = order+[var for var in columns if var not in order]
+            index = list(self.devices.keys())
+        elif dev_type == "varactor":
+            for varactor in self.varactors.values():
+                for i,var in enumerate(vars):
+                    data[columns[i]].append(getattr(varactor, var))
+            order = ["name", "type", "vgs", "l", "w", "cvar"]
+            order = [var+"["+MosCell.__UNITS__[var]+"]" for var in order]
+            order = order+[var for var in columns if var not in order]
+            index = self.varactors.keys()
+        elif dev_type == "switch":
+            for switch in self.switches.values():
+                for i,var in enumerate(vars):
+                    data[columns[i]].append(getattr(switch, var))
+            order = ["name", "type", "vgs", "l", "w", "ron"]
+            order = [var+"["+MosCell.__UNITS__[var]+"]" for var in order]
+            order = order+[var for var in columns if var not in order]
+            index = self.switches.keys()
+        else:
+            raise ValueError(f"Unknown device type: {dev_type}")
+        return DataFrame(data, index=index)[order]
 
     def __str__(self) -> str:
-        return str(self.__data_frame__())
+        devices_df = self.__data_frame__(dev_type = "cell")
+        vars_df = self.__data_frame__(dev_type = "varactor")
+        swtchs_df = self.__data_frame__(dev_type = "switch")
+        ret = "Devices:\n"
+        ret += str(devices_df)+"\n" if not devices_df.empty else "--None--\n"
+        ret += "Varactors:\n"
+        ret += str(vars_df)+"\n" if not vars_df.empty else "--None--\n"
+        ret += "Switches:\n"
+        ret += str(swtchs_df)+"\n" if not swtchs_df.empty else "--None--\n"
+        return ret
     
-    def add(self, device) -> None:
-        self.devices[device.name] = device
+    def add(self, device, dev_type = "cell") -> None:
+        """_summary_
+        Adds a new device to the correct database of
+        parsed devices
+        Args:
+            device (_type_): _description_
+            dev_type (str, optional): Device type identifier. Defaults to "cell".
+
+        Raises:
+            KeyError: _description_
+            ValueError: _description_
+        """
+        if device.name in list(self.devices.keys()) + list(self.varactors.keys()) + list(self.switches.keys()):
+            raise KeyError(f"A device with the name \"{device.name}\" was already parsed.")
+        if dev_type == "varactor":
+            self.varactors[device.name] = device
+        elif dev_type == "cell":
+            self.devices[device.name] = device
+        elif dev_type == "switch":
+            self.switches[device.name] = device
+        else:
+            raise ValueError(f"Unknown device type: {dev_type}")
 
     def parse_data(self, data:dict) -> None:
         """_summary_
@@ -341,7 +449,19 @@ class Devices(object):
                     device = MosCell(name=token)
                     for key,token in list(data[sec].items())[1:]:
                         device.__parse_data__(key, token)
-                    self.add(device)
+                    self.add(device, dev_type="cell")
+                elif key == TomlControlKeywords.VARACTOR.value:
+                    # create a new varactor
+                    varactor = MosCell(name=token)
+                    for key,token in list(data[sec].items())[1:]:
+                        varactor.__parse_data__(key, token)
+                    self.add(varactor, dev_type="varactor")
+                elif key == TomlControlKeywords.SWITCH.value:
+                    # create a new switch
+                    switch = MosCell(name=token)
+                    for key,token in list(data[sec].items())[1:]:
+                        switch.__parse_data__(key, token)
+                    self.add(switch, dev_type="switch")
                 elif key == TomlControlKeywords.DEVICES.value:
                     devices_names = [n for n in token]
                     for key,token in list(data[sec].items())[1:]:
@@ -351,6 +471,24 @@ class Devices(object):
                         for subkey, subtoken in token.items():
                             device.__parse_data__(subkey, subtoken)
                         self.add(device)
+                elif key == TomlControlKeywords.VARACTORS.value:
+                    varactors_names = [n for n in token]
+                    for key,token in list(data[sec].items())[1:]:
+                        if key not in varactors_names:
+                            raise ValueError(f"{key} is an unrecognized device name")
+                        varactor = MosCell(name=key)
+                        for subkey, subtoken in token.items():
+                            varactor.__parse_data__(subkey, subtoken)
+                        self.add(varactor, dev_type="varactor")
+                elif key == TomlControlKeywords.SWITCHES.value:
+                    switches_names = [n for n in token]
+                    for key,token in list(data[sec].items())[1:]:
+                        if key not in switches_names:
+                            raise ValueError(f"{key} is an unrecognized device name")
+                        switch = MosCell(name=key)
+                        for subkey, subtoken in token.items():
+                            switch.__parse_data__(subkey, subtoken)
+                        self.add(switch, dev_type="switch")
                 else:
                     raise ValueError(f"{key} is not a valid key for the first argument of {TomlSections.CONTROL.name} section. First argument must be {TomlControlKeywords.DEVICE.name} or {TomlControlKeywords.DEVICES.name}.")
             elif sec == TomlSections.SPIT.value:
@@ -373,8 +511,8 @@ class Devices(object):
                         [acceptable_vars.append(v) for v in dir(MosCell) if not v.startswith('__')]
                         for subkey, subtoken in token.items():
                             
-                            if subkey not in list(self.devices.keys()):
-                                raise ValueError(f"{subkey} is an unrecognized device name. Recognized devices are {list(self.devices.keys())}")
+                            if subkey not in list(self.devices.keys()) + list(self.varactors.keys()) + list(self.switches.keys()):
+                                raise ValueError(f"{subkey} is an unrecognized device name. Recognized devices are {list(self.devices.keys()) + list(self.varactors.keys()) + list(self.switches.keys())}")
                             for var in subtoken:    
                                 if var not in acceptable_vars:
                                     raise ValueError(f"{var} is an unrecognized variable name")
@@ -382,5 +520,5 @@ class Devices(object):
                                     self.spits[subkey].append(var)
             else:
                 raise ValueError(f"{sec} is not a valid section name")
-        if len(self.devices) == 0:
+        if len(self.devices) == 0 and len(self.varactors) == 0 and len(self.switches) == 0:
             raise ValueError("No devices were detected in the TOML setup file")
\ No newline at end of file
